# -*- coding: utf-8 -*-
from .connectionmanagement import ConnectionManagement

cache_app = ConnectionManagement()

class Touch():

    def __init__(self):
        #เนื่องจากปัญหาเรื่องโครงสร้าง structure เลยยังไม่สามารถใช้ได้
        pass


    def native_tap_with_positions(self, duration=500, *locations):
        """Taps on a particular place with up to five fingers, holding for a
        certain time

        Args:
        - locations - an array of tuples representing the x/y coordinates of
                the fingers to tap. Length can be up to five.
        - duration - length of time to tap, in ms. Default: 500ms

        Example:
        |  @{firstFinger}   |  create list  |  ${100}  |  ${500}  |
        |  @{secondFinger}  |  create list  |${700}    |  ${500}  |
        |  @{fingerPositions}  |  create list  |  ${firstFinger}  |  ${secondFinger}  |
        |  Sleep  |  1  |
        |  Tap with Positions  |  ${1000}  |  @{fingerPositions}  |

        ============================================================

        แตะบนจุดที่ระบุด้วยนิ้วมือสูงสุดถึงห้านิ้ว โดยกดค้างไว้เป็นระยะเวลาหนึ่ง

        อาร์กิวเมนต์:
        - locations - อาร์เรย์ของทูเพิลแสดงพิกัด x/y ของนิ้วที่จะแตะ ความยาวสามารถเป็นได้ถึงห้า
        - duration - ระยะเวลาในการแตะ, เป็นมิลลิวินาที ค่าเริ่มต้น: 500 มิลลิวินาที

        ตัวอย่าง:
        |  @{firstFinger}   |  สร้างรายการ  |  ${100}  |  ${500}  |
        |  @{secondFinger}  |  สร้างรายการ  |${700}    |  ${500}  |
        |  @{fingerPositions}  |  สร้างรายการ  |  ${firstFinger}  |  ${secondFinger}  |
        |  หลับ  |  1  |
        |  แตะด้วยตำแหน่ง  |  ${1000}  |  @{fingerPositions}  |
        """
        driver = cache_app._current_application()
        driver.tap(positions=list(locations), duration=duration)


# -*- coding: utf-8 -*-

import os
import robot

from .connectionmanagement import ConnectionManagement
from AppiumLibrary.keywords._logging import _LoggingKeywords
from AppiumFlutterLibrary.keywords._screen import _ScreenKeywords

cache_app = ConnectionManagement()
log = _LoggingKeywords()
screen_instance = _ScreenKeywords()

class CaptureScreenShot:

    def __init__(self):
        self._screenshot_index = 0
    
    #KeyWord

    def native_capture_page_screenshot(self, filename=None):
        """Takes a screenshot of the current page and embeds it into the log.

        `filename` argument specifies the name of the file to write the
        screenshot into. If no `filename` is given, the screenshot will be
        embedded as Base64 image to the log.html. In this case no file is created in the filesystem.

        Warning: this behavior is new in 1.7. Previously if no filename was given
        the screenshots where stored as separate files named `appium-screenshot-<counter>.png`

        Example:

        |t_capture_page_screenshot | None |

        =========================================================

        ถ่ายภาพหน้าจอของหน้าปัจจุบันและฝังลงในบันทึก

        อาร์กิวเมนต์ filename ระบุชื่อของไฟล์ที่จะเขียนภาพหน้าจอลงไป หากไม่มีการระบุ filename, ภาพหน้าจอจะถูกฝังเป็นรูปภาพ Base64 ลงใน log.html
        ในกรณีนี้ไม่มีไฟล์ใดถูกสร้างขึ้นในระบบไฟล์

        คำเตือน: พฤติกรรมนี้เป็นใหม่ในเวอร์ชัน 1.7 ก่อนหน้านี้หากไม่มีการระบุชื่อไฟล์ ภาพหน้าจอจะถูกเก็บเป็นไฟล์แยกต่างหากชื่อว่า 
        appium-screenshot-<counter>.png

        ตัวอย่างการใช้งาน:

        |t_capture_page_screenshot | None |

        แก้กลับเป็น ver 1.4 ก่อน เพราะต้องใช้รูปใน line noti
        """
        path, link = self._get_screenshot_paths(filename)

        if hasattr(cache_app._current_application(), 'get_screenshot_as_file'):
            cache_app._current_application().get_screenshot_as_file(path)
        else:
            cache_app._current_application().save_screenshot(path)

        # Image is shown on its own row and thus prev row is closed on purpose
        log._html('</td></tr><tr><td colspan="3"><a href="%s">'
                   '<img src="%s" width="800px"></a>' % (link, link))
        return path

        #แก้กลับเป็น ver 1.4 ก่อน เพราะต้องใช้รูปใน line noti
        # if filename:
        #     path, link = self._get_screenshot_paths(filename)

        #     if hasattr(cache_app._current_application(), 'get_screenshot_as_file'):
        #         cache_app._current_application().get_screenshot_as_file(path)
        #     else:
        #         cache_app._current_application().save_screenshot(path)

        #     # Image is shown on its own row and thus prev row is closed on purpose
        #     log._html('</td></tr><tr><td colspan="3"><a href="%s">'
        #                '<img src="%s" width="800px"></a>' % (link, link))
        #     return path
        # else:
        #     base64_screenshot = cache_app._current_application().get_screenshot_as_base64()
        #     log._html('</td></tr><tr><td colspan="3">'
        #                '<img src="data:image/png;base64, %s" width="800px">' % base64_screenshot)
        #     return None

    #Private_Function
        
    def _get_screenshot_paths(self, filename):
        if not filename:
            screenshot_index_value = getattr(screen_instance, '_screenshot_index', None)
            print('getattr')
            print(screenshot_index_value)
            self._screenshot_index = screenshot_index_value+1
            filename = 'appiumflutter-screenshot-%d.png' % self._screenshot_index
            setattr(screen_instance, '_screenshot_index', self._screenshot_index)
            print('setattr')
            print(self._screenshot_index)
        else:
            filename = filename.replace('/', os.sep)
        logdir = self._get_log_dir()
        path = os.path.join(logdir, filename)
        link = robot.utils.get_link_path(path, logdir)
        return path, link




# -*- coding: utf-8 -*-
# from appium_flutter_finder import FlutterElement , FlutterFinder
from AppiumLibrary import utils
from appium.webdriver.common.appiumby import AppiumBy
from robot.api import logger
from TineAutomationToolkit.detect.flutterfinderwidget import FlutterDetectWidget , FlutterFinderWidget

detect_element = FlutterFinderWidget()
# strategies = {
#     'default': _find_by_default,
#     'id': _find_by_id,
#     'key': _find_by_key,
#  }


class DetectElement():

    def __init__(self):
        #เนื่องจากปัญหาเรื่องโครงสร้าง structure เลยยังไม่สามารถใช้ได้
        self._strategies = {
            'default': self._find_by_default,
            'key': self._find_by_key,
            'xpath': self._find_by_xpath,
            'id': self._find_by_id,
        }
        # pass

    def find_attribute(self, application , locator , tag=None):
        print("Find Attribute Locator")
        assert application is not None
        assert locator is not None and len(locator) > 0

        (prefix, criteria) = self._parse_locator(locator)
        print("Data Prefix & Criteria")
        print(prefix)
        print(criteria)

        prefix = 'default' if prefix is None else prefix
        # strategy = strategies.get(prefix)
        strategy = self._strategies.get(prefix)
        print(strategy)

        if strategy is None:
            raise ValueError("Element locator with prefix '" + prefix + "' is not supported")
        
        (tag, constraints) = self._get_tag_and_constraints(tag)
        
        return strategy(application, criteria, tag, constraints)

    def _find_by_default(self, application, criteria, tag, constraints):
        if criteria.startswith('//'):
            return self._find_by_xpath(application, criteria, tag, constraints)
        # Used `id` instead of _find_by_key_attrs since iOS and Android internal `id` alternatives are
        # different and inside appium python client. Need to expose these and improve in order to make
        # _find_by_key_attrs useful.
        # return self._find_by_id(application, criteria, tag, constraints)
        return self._find_by_key(application, criteria, tag, constraints)
    
    def _find_by_key(self, application, element_key, tag, constraints):
        finder_key = detect_element.by_value_key(element_key)
        element = FlutterDetectWidget(application, finder_key)

        return element
    
    def _find_by_xpath(self, application, criteria, tag, constraints):
        print(f"xpath criteria: {criteria}")
        return self._filter_elements(
            application.find_elements(by=AppiumBy.XPATH, value=criteria),
            tag, constraints)
    
    def _find_by_id(self, application, criteria, tag, constraints):
        print(f"criteria is {criteria}")
        return self._filter_elements(
            application.find_elements(by=AppiumBy.ID, value=criteria),
            tag, constraints)

    #PRAVITE_FUNCTION

    def _parse_locator(self, locator):
        prefix = None
        criteria = locator
        if not locator.startswith('//'):
            locator_parts = locator.partition('=')
            if len(locator_parts[1]) > 0:
                prefix = locator_parts[0].strip().lower()
                criteria = locator_parts[2].strip()
        return (prefix, criteria)


    def _get_tag_and_constraints(self, tag):
        if tag is None:
            return None, {}

        tag = tag.lower()
        constraints = {}
        if tag == 'link':
            tag = 'a'
        elif tag == 'image':
            tag = 'img'
        elif tag == 'list':
            tag = 'select'
        elif tag == 'radio button':
            tag = 'input'
            constraints['type'] = 'radio'
        elif tag == 'checkbox':
            tag = 'input'
            constraints['type'] = 'checkbox'
        elif tag == 'text field':
            tag = 'input'
            constraints['type'] = 'text'
        elif tag == 'file upload':
            tag = 'input'
            constraints['type'] = 'file'
        return tag, constraints
    
    def _filter_elements(self, elements, tag, constraints):
        elements = self._normalize_result(elements)
        if tag is None:
            return elements
        return filter(
            lambda element: self._element_matches(element, tag, constraints),
            elements)
    
    def _element_matches(self, element, tag, constraints):
        if not element.tag_name.lower() == tag:
            return False
        for name in constraints:
            if not element.get_attribute(name) == constraints[name]:
                return False
        return True
    
    def _normalize_result(self, elements):
        if not isinstance(elements, list):
            logger.debug("WebDriver find returned %s" % elements)
            return []
        return elements